# Botan base spec file

# Note that some of the commands in here pretty much assume a GNU
# toolset, which is unfortunate and should probably be fixed.

##################################################
# Hardware restrictions on various modules       #
##################################################
%define HAS_RDTSC i586 i686 athlon

##################################################
# Version numbers and other config options       #
##################################################
%define MAJOR $MAJOR
%define MINOR $MINOR
%define PATCH $PATCH

# This is the list of modules built into the library. We also
# include the 'unix' module set.
%define THREADS      mutex_pthread
%define MISC         comp_bzip2,comp_zlib

%ifarch %{HAS_RDTSC}
  %define TIMERS       timer_posix,timer_hw_ia32
%endif

%ifnarch %{HAS_RDTSC}
  %define TIMERS       timer_posix
%endif

%define MODULES %{TIMERS},%{THREADS},%{MISC}

##################################################
# Descriptions                                   #
##################################################
%define VERSION %{MAJOR}.%{MINOR}.%{PATCH}

Name: Botan
Summary: A C++ crypto library
Version: %{VERSION}
Release: 1
Copyright: BSD
Group: System Environment/Libraries
Requires: bzip2 >= 1.0.0, zlib >= 1.1.4
Source: http://botan.randombit.net/botan/files/Botan-%{VERSION}.tgz
URL: http://botan.randombit.net
Packager: Jack Lloyd <lloyd@randombit.net>
BuildRoot: /var/tmp/botan-rpmroot
Prefix: /usr/local

%description
Botan is a library, written in C++, for providing cryptographic algorithms to
any application which may find such things useful. It includes a wide selection
of cryptographic algorithms, and a high-level filter based interface.

%package devel
Summary: Development files for Botan
Group: Development/Libraries
Requires: Botan = %{VERSION}
%description devel
This package contains the header files and libraries needed to develop programs
that use the Botan library.

##################################################
# Main Logic                                     #
##################################################
%prep
%setup -n Botan-%{VERSION}

%build
./configure.pl --module-set=unix --modules=%{MODULES} \
  gcc-%{_target_os}-%{_target_cpu}
# Work around the occasionally somewhat-less-than-functional RedHat compiler
make LIB_OPT="-O3" shared static

%clean
rm -rf $RPM_BUILD_ROOT

%install
ROOT="$RPM_BUILD_ROOT/usr/local"
make OWNER=`id -u` GROUP=`id -g` INSTALLROOT="$ROOT" install

# Need this since we're installing shared libs...
%post
if ! grep "^$RPM_INSTALL_PREFIX/lib$" /etc/ld.so.conf 2>&1 >/dev/null
then
   echo "$RPM_INSTALL_PREFIX/lib" >>/etc/ld.so.conf
fi
/sbin/ldconfig

%post devel

%postun
RMDIR_IGNORE_NONEMPTY="rmdir --ignore-fail-on-non-empty"
/sbin/ldconfig
if [ -d $RPM_INSTALL_PREFIX/share/doc/botan-%{VERSION} ]; then
   $RMDIR_IGNORE_NONEMPTY $RPM_INSTALL_PREFIX/share/doc/botan-%{VERSION}
fi

%postun devel
RMDIR_IGNORE_NONEMPTY="rmdir --ignore-fail-on-non-empty"
if [ -d $RPM_INSTALL_PREFIX/include/botan ]; then
   $RMDIR_IGNORE_NONEMPTY $RPM_INSTALL_PREFIX/include/botan
fi

##################################################
# File Lists                                     #
##################################################
%files
%attr(-,root,root) %docdir /usr/local/share/doc/botan-%{VERSION}/
%attr(-,root,root) /usr/local/share/doc/botan-%{VERSION}/license.txt
%attr(-,root,root) /usr/local/share/doc/botan-%{VERSION}/readme.txt
%attr(-,root,root) /usr/local/share/doc/botan-%{VERSION}/log.txt
%attr(-,root,root) /usr/local/share/doc/botan-%{VERSION}/thanks.txt
%attr(-,root,root) /usr/local/share/doc/botan-%{VERSION}/authors.txt
%attr(-,root,root) /usr/local/share/doc/botan-%{VERSION}/pgpkeys.asc
%attr(-,root,root) /usr/local/lib/libbotan-%{MAJOR}.so
%attr(-,root,root) /usr/local/lib/libbotan-%{MAJOR}.%{MINOR}.so
%attr(-,root,root) /usr/local/lib/libbotan-%{MAJOR}.%{MINOR}.%{PATCH}.so

%files devel
%attr(-,root,root) %docdir /usr/local/share/doc/botan-%{VERSION}/
%attr(-,root,root) /usr/local/share/doc/botan-%{VERSION}/bugs.txt
%attr(-,root,root) /usr/local/share/doc/botan-%{VERSION}/deprecated.txt
%attr(-,root,root) /usr/local/share/doc/botan-%{VERSION}/todo.txt
%attr(-,root,root) /usr/local/share/doc/botan-%{VERSION}/indent.el
%attr(-,root,root) /usr/local/share/doc/botan-%{VERSION}/api.ps
%attr(-,root,root) /usr/local/share/doc/botan-%{VERSION}/api.tex
%attr(-,root,root) /usr/local/lib/libbotan.so
%attr(-,root,root) /usr/local/lib/libbotan.a
%attr(-,root,root) /usr/local/include/botan/
%attr(-,root,root) /usr/local/bin/botan-config
