/*
Decrypt (and verify) an encrypted DSA private key. Then use that key to sign a
message.

Written by Jack Lloyd (lloyd@randombit.net), August 5, 2002
   Updated to use X.509 and PKCS #8 format keys, October 21, 2002

This file is in the public domain
*/

#include <iostream>
#include <iomanip>
#include <fstream>
#include <cstdlib>
#include <string>

#include <botan/filters.h>
#include <botan/look_pk.h>
#include <botan/lookup.h>
#include <botan/dsa.h>
using namespace Botan;
using namespace std;

const std::string SUFFIX = ".sig";

int main(int argc, char* argv[])
   {
   try {

   if(argc != 4)
      {
      cout << "Usage: dsa_sig keyfile messagefile passphrase" << endl;
      return 1;
      }

   ifstream keyfile(argv[1]);
   if(!keyfile)
      {
      cout << "Couldn't read the key file." << endl;
      return 1;
      }

   LibraryInitializer init;

   std::string passphrase(argv[3]);

   ifstream message(argv[2]);
   if(!message)
      {
      cout << "Couldn't read the message file." << endl;
      return 1;
      }

   DataSource_Stream keystream(keyfile);
   DSA_PrivateKey dsakey(keystream, passphrase);

   std::string outfile = argv[2] + SUFFIX;
   ofstream sigfile(outfile.c_str());
   if(!sigfile)
      {
      cout << "Couldn't write the signature to " << outfile << endl;
      return 1;
      }

   Pipe pipe(new PK_Signer_Filter(get_pk_signer(dsakey, "EMSA1(SHA-1)")),
             new Base64_Encoder);

   pipe.start_msg();
   message >> pipe;
   pipe.end_msg();

   sigfile << pipe.read_all_as_string() << std::endl;
   }
   catch(exception& e)
      {
      cout << "Exception caught: " << e.what() << endl;
      }
   return 0;
   }
