/*
  Read in an X.509 certificate, and print various things about it

  Written by Jack Lloyd, March 23 2003

  This file is in the public domain
*/
#include <botan/x509cert.h>
#include <botan/oids.h>
using namespace Botan;

#include <iostream>

void do_print(const std::string& what, const std::string& val)
   {
   if(val == "")
      return;

   std::cout << "   " << what << ": " << val << std::endl;
   }

void do_subject(const X509_Certificate& cert, const std::string& what)
   {
   do_print(what, cert.subject_info(what));
   }

void do_issuer(const X509_Certificate& cert, const std::string& what)
   {
   do_print(what, cert.issuer_info(what));
   }

int main(int argc, char* argv[])
   {
   if(argc != 2 && argc != 3)
      {
      std::cout << "Usage: " << argv[0] << " [-d] <x509cert>\n";
      return 1;
      }

   LibraryInitializer init;

   X509_Encoding code = PEM;

   std::string filename;

   if(strcmp(argv[1], "-d") == 0 && argc == 3)
      {
      code = RAW_BER;
      filename = argv[2];
      }
   else if(argc == 2)
      filename = argv[1];
   else
      {
      std::cout << "Bad args\n";
      return 1;
      }

   try {
      DataSource_Stream source(filename);

      X509_Certificate cert(source, code);

      std::cout << "Subject" << std::endl;
      do_subject(cert, "Name");
      do_subject(cert, "Email");
      do_subject(cert, "Organization");
      do_subject(cert, "Organizational Unit");
      do_subject(cert, "Locality");
      do_subject(cert, "State");
      do_subject(cert, "Country");

      std::cout << "Issuer" << std::endl;
      do_issuer(cert, "Name");
      do_issuer(cert, "Email");
      do_issuer(cert, "Organization");
      do_issuer(cert, "Organizational Unit");
      do_issuer(cert, "Locality");
      do_issuer(cert, "State");
      do_issuer(cert, "Country");

      std::cout << "Validity" << std::endl;

      std::cout << "   Not before: " << cert.start_time() << std::endl;
      std::cout << "   Not after: " << cert.end_time() << std::endl;

      std::cout << "Signature algorithm: " <<
         OIDS::lookup(cert.signature_algorithm().oid) << std::endl;

   }
   catch(std::exception& e)
      {
      std::cout << e.what() << std::endl;
      return 1;
      }
   return 0;
   }
