/*************************************************
* DSA Header File                                *
* (C) 1999-2003 The Botan Project                *
*************************************************/

#ifndef BOTAN_DSA_H__
#define BOTAN_DSA_H__

#include <botan/dl_algo.h>
#include <botan/mod_exp.h>

namespace Botan {

/*************************************************
* DSA Public Key                                 *
*************************************************/
class DSA_PublicKey : public PK_Verifying_wo_MR_Key,
                      public virtual DL_Scheme_PublicKey
   {
   public:
      bool verify(const byte[], u32bit, const byte[], u32bit) const;
      u32bit max_input_bits() const;

      DSA_PublicKey(const DL_Group&, const BigInt&);
      DSA_PublicKey(DataSource&, X509_Encoding = PEM);
   protected:
      DSA_PublicKey() {}
      FixedBase_Exp powermod_g_p, powermod_y_p;
   private:
      friend X509_PublicKey* get_public_key(const std::string&);
      u32bit message_parts() const { return 2; }
      u32bit message_part_size() const;
      SecureVector<byte> DER_encode_params() const;
      void BER_decode_pub(DataSource&);
      void BER_decode_params(DataSource&);
      std::string algo_name() const;
   };

/*************************************************
* DSA Private Key                                *
*************************************************/
class DSA_PrivateKey : public DSA_PublicKey, public PK_Signing_Key,
                       public virtual DL_Scheme_PrivateKey
   {
   public:
      SecureVector<byte> sign(const byte[], u32bit) const;

      bool check_params() const;

      DSA_PrivateKey(const DL_Group&);
      DSA_PrivateKey(const DL_Group&, const BigInt&, const BigInt&);
      DSA_PrivateKey(DataSource&, const std::string& = "",
                     X509_Encoding = PEM);
   private:
      friend PKCS8_PrivateKey* get_private_key(const std::string&);
      void BER_decode_priv(DataSource&);
      DSA_PrivateKey() {}
   };

}

#endif
