/*************************************************
* X.509 Certificate Authority Header File        *
* (C) 1999-2003 The Botan Project                *
*************************************************/

#ifndef BOTAN_X509_CA_H__
#define BOTAN_X509_CA_H__

#include <botan/pkcs8.h>
#include <botan/pubkey.h>
#include <botan/x509cert.h>
#include <botan/x509_crl.h>
#include <botan/x509opt.h>
#include <map>

namespace Botan {

/*************************************************
* X.509 Certificate Authority                    *
*************************************************/
class X509_CA
   {
   public:
      X509_Certificate sign_key(const X509_PublicKey&,
                                const X509_Cert_Options&) const;

      X509_Certificate ca_certificate() const;

      X509_CRL new_crl() const;
      X509_CRL update_crl(const std::vector<CRL_Entry>&,
                          const X509_CRL&) const;

      X509_CA(const X509_Certificate&, const PKCS8_PrivateKey&);
      X509_CA(const PKCS8_PrivateKey&, X509_Cert_Options&);
      ~X509_CA();
   private:
      X509_CA(const X509_CA&) {}
      X509_CA& operator=(const X509_CA&) { return (*this); }

      X509_Certificate make_cert(const SecureVector<byte>&,
                                 const X509_Cert_Options&,
                                 Key_Constraints) const;
      X509_CRL make_crl(const std::vector<CRL_Entry>&, u32bit) const;
      void do_ext(DER_Encoder&, DER_Encoder&,
                  const std::string&, const std::string&) const;

      AlgorithmIdentifier ca_sig_algo;
      X509_Certificate cert;
      PK_Signer* signer;
   };

}

#endif
