/*************************************************
* X.509 Certificate Options Header File          *
* (C) 1999-2003 The Botan Project                *
*************************************************/

#ifndef BOTAN_X509_CERT_OPTS_H__
#define BOTAN_X509_CERT_OPTS_H__

#include <botan/asn1_obj.h>

namespace Botan {

/*************************************************
* Options for X.509 Certificates                 *
*************************************************/
class X509_Cert_Options
   {
   public:
      void set_name(const std::string&);
      void set_country(const std::string&);
      void set_org(const std::string&);
      void set_org_unit(const std::string&);
      void set_locality(const std::string&);
      void set_state(const std::string&);

      void set_email(const std::string&);
      void set_dns_name(const std::string&);
      void set_uri(const std::string&);

      void set_validity_start(const std::string&);
      void set_validity_end(const std::string&);

      void set_constraints(Key_Constraints);
      void add_ex_constraints(const OID&);

      void CA_key(u32bit = 8);

      X509_Cert_Options(const std::string& = "");
   private:
      friend class X509_CA;
      void sanity_check() const;
      std::string name, country, org, org_unit, locality, state;
      std::string email, uri, dns;
      X509_Time start, end;
      Key_Constraints constraints;
      std::vector<OID> ex_constraints;
      u32bit path_limit;
      bool is_ca, self_signed;
   };

}

#endif
