/*************************************************
* Zlib Compressor Header File                    *
* (C) 1999-2003 The Botan Project                *
*************************************************/

#include <botan/filter.h>

#ifndef BOTAN_EXT_ZLIB_H__
#define BOTAN_EXT_ZLIB_H__

namespace Botan {

/*************************************************
* Zlib Compression Filter                        *
*************************************************/
class Zlib_Compression : public Filter
   {
   public:
      void write(const byte input[], u32bit length);
      void start_msg();
      void end_msg();

      void flush();

      Zlib_Compression(u32bit l = 6) : level((l >= 9) ? 9 : l) { zlib = 0; }
      ~Zlib_Compression() { clear(); }
   private:
      static const u32bit BUFFERSIZE = DEFAULT_BUFFERSIZE;
      void clear();
      const u32bit level;
      SecureBuffer<byte, BUFFERSIZE> buffer;
      struct zlib_stream_wrapper* zlib;
   };

/*************************************************
* Zlib Decompression Filter                      *
*************************************************/
class Zlib_Decompression : public Filter
   {
   public:
      void write(const byte input[], u32bit length);
      void start_msg();
      void end_msg();

      Zlib_Decompression() { zlib = 0; no_writes = true; }
      ~Zlib_Decompression() { clear(); }
   private:
      static const u32bit BUFFERSIZE = DEFAULT_BUFFERSIZE;
      void clear();
      SecureBuffer<byte, BUFFERSIZE> buffer;
      struct zlib_stream_wrapper* zlib;
      bool no_writes;
   };

}

#endif
