/*************************************************
* Pthreads Entropy Source Source File            *
* (C) 1999-2003 The Botan Project                *
*************************************************/

#define _POSIX_C_SOURCE 199506
#include <botan/es_pthr.h>
#include <botan/util.h>
#include <pthread.h>
#include <time.h>

namespace Botan {

namespace {

/*************************************************
* Thread arguments structure                     *
*************************************************/
struct timer_arg
   {
   u64bit* timer;
   pthread_mutex_t* mutex;
   };

/*************************************************
* Compete with other threads for the mutex       *
*************************************************/
void* timer(void* arg_pointer)
   {
   const u32bit LOCKS = 4;

   timer_arg* arg = static_cast<timer_arg*>(arg_pointer);
   u32bit locks_gained = 0;

   while(locks_gained < LOCKS)
      {
      timespec sleep_time;
      sleep_time.tv_sec = 0;
      sleep_time.tv_nsec = 1000;
      nanosleep(&sleep_time, 0);

      pthread_mutex_lock(arg->mutex);
      *(arg->timer) += system_clock();
      pthread_mutex_unlock(arg->mutex);
      locks_gained++;
      }
   return 0;
   }

}

/*************************************************
* Gather entropy from thread jitter              *
*************************************************/
void Pthread_EntropySource::do_fast_poll()
   {
   do_poll(16);
   }

/*************************************************
* Gather entropy from thread jitter              *
*************************************************/
void Pthread_EntropySource::do_slow_poll()
   {
   do_poll(32);
   }

/*************************************************
* Gather entropy from thread jitter              *
*************************************************/
void Pthread_EntropySource::do_poll(u32bit timings)
   {
   for(u32bit j = 0; j != timings; j++)
      {
      u64bit timing = get_timing();
      add_bytes(&timing, sizeof(u64bit));
      }
   }

/*************************************************
* Sample the thread-based timer jitter           *
*************************************************/
u64bit Pthread_EntropySource::get_timing()
   {
   const u32bit THREAD_COUNT = 16;

   pthread_t threads[THREAD_COUNT];
   u64bit timers[THREAD_COUNT] = { 0 };
   pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
   timer_arg args[THREAD_COUNT];

   for(u32bit j = 0; j != THREAD_COUNT; j++)
      {
      args[j].timer = &timers[j];
      args[j].mutex = &mutex;
      pthread_create(&threads[j], 0, timer, &args[j]);
      }
   for(u32bit j = 0; j != THREAD_COUNT; j++)
      pthread_join(threads[j], 0);

   u64bit timer_sum = 0;
   for(u32bit j = 0; j != THREAD_COUNT; j++)
      timer_sum ^= timers[j];
   return timer_sum;
   }

}
