/*************************************************
* Mutex Factory Source File                      *
* (C) 1999-2003 The Botan Project                *
*************************************************/

#include <botan/mutex.h>
#include <botan/exceptn.h>

namespace Botan {

namespace {

Mutex* mutex_factory = 0;
Mutex* mutex_init_lock = 0;

}

/*************************************************
* Default Mutex                                  *
*************************************************/
class Default_Mutex : public Mutex
   {
   public:
      void lock();
      void unlock();
      Mutex* clone() const { return new Default_Mutex; }
      Default_Mutex() { locked = false; }
   private:
      bool locked;
   };

/*************************************************
* Lock the Mutex                                 *
*************************************************/
void Default_Mutex::lock()
   {
   if(locked)
      throw Internal_Error("Default_Mutex::lock: Mutex is already locked");
   locked = true;
   }

/*************************************************
* Unlock the Mutex                               *
*************************************************/
void Default_Mutex::unlock()
   {
   if(!locked)
      throw Internal_Error("Default_Mutex::unlock: Mutex is already unlocked");
   locked = false;
   }

/*************************************************
* Get a Mutex                                    *
*************************************************/
Mutex* get_mutex()
   {
   if(mutex_factory == 0)
      mutex_factory = new Default_Mutex;
   return mutex_factory->clone();
   }

/*************************************************
* Initialize a mutex atomically                  *
*************************************************/
void initialize_mutex(Mutex*& mutex)
   {
   mutex_init_lock->lock();
   if(mutex == 0)
      mutex = get_mutex();
   mutex_init_lock->unlock();
   }

/*************************************************
* Set the Mutex type                             *
*************************************************/
void set_mutex_type(Mutex* mutex)
   {
   if(mutex == 0) return;
   delete mutex_factory;
   mutex_factory = mutex;
   delete mutex_init_lock;
   mutex_init_lock = get_mutex();
   }

}
