/*************************************************
* OID Registry Source File                       *
* (C) 1999-2003 The Botan Project                *
*************************************************/

#include <botan/oids.h>
#include <botan/exceptn.h>
#include <map>

namespace Botan {

namespace OIDS {

namespace {

std::map<OID, std::string> oid_to_str;
std::map<std::string, OID> str_to_oid;

}

/*************************************************
* Register an OID to string mapping              *
*************************************************/
void add_oid(const OID& oid, const std::string& name, bool is_alias)
   {
   str_to_oid[name] = oid;
   if(!is_alias)
      oid_to_str[oid] = name;
   }

/*************************************************
* Do an OID to string lookup                     *
*************************************************/
std::string lookup(const OID& oid)
   {
   std::map<OID, std::string>::const_iterator info = oid_to_str.find(oid);
   if(info == oid_to_str.end())
      return oid.as_string();
   return info->second;
   }

/*************************************************
* Do a string to OID lookup                      *
*************************************************/
OID lookup(const std::string& name)
   {
   std::map<std::string, OID>::const_iterator info = str_to_oid.find(name);
   if(info == str_to_oid.end())
      throw Lookup_Error("No known OID for " + name);
   return info->second;
   }

/*************************************************
* Check to see if an OID exists in the table     *
*************************************************/
bool have_oid(const std::string& name)
   {
   std::map<std::string, OID>::const_iterator info = str_to_oid.find(name);
   return (info != str_to_oid.end());
   }

}

}
