/*************************************************
* PEM Encoding/Decoding Source File              *
* (C) 1999-2003 The Botan Project                *
*************************************************/

#include <botan/pem.h>
#include <botan/filters.h>

namespace Botan {

/*************************************************
* PEM encode BER/DER-encoded objects             *
*************************************************/
std::string pem_encode(const byte der[], u32bit length,
                       const std::string& label)
   {
   const std::string PEM_HEADER = "-----BEGIN " + label + "-----\n";
   const std::string PEM_TRAILER = "-----END " + label + "-----\n";

   Pipe pipe(new Base64_Encoder(true, 64));
   pipe.process_msg(der, length);
   return (PEM_HEADER + pipe.read_all_as_string() + PEM_TRAILER);
   }

/*************************************************
* Decode PEM down to raw BER/DER                 *
*************************************************/
SecureVector<byte> pem_decode_check_label(DataSource& source,
                                          const std::string& label_want)
   {
   std::string label_got;
   SecureVector<byte> ber = pem_decode(source, label_got);
   if(label_got != label_want)
      throw Decoding_Error("PEM: Label mismatch, wanted " + label_want +
                           ", got " + label_got);
   return ber;
   }

/*************************************************
* Decode PEM down to raw BER/DER                 *
*************************************************/
SecureVector<byte> pem_decode(DataSource& source, std::string& label)
   {
   const std::string PEM_HEADER1 = "-----BEGIN ";
   const std::string PEM_HEADER2 = "-----";
   u32bit position = 0;
   while(position != PEM_HEADER1.length())
      {
      byte b;
      if(!source.read_byte(b))
         throw Decoding_Error("PEM: No PEM header found");
      if(b == PEM_HEADER1[position])
         position++;
      else if(position)
         throw Decoding_Error("PEM: Malformed PEM");
      }
   position = 0;
   while(position != PEM_HEADER2.length())
      {
      byte b;
      if(!source.read_byte(b))
         throw Decoding_Error("PEM: No PEM header found");
      if(b == PEM_HEADER2[position])
         position++;
      else if(position)
         throw Decoding_Error("PEM: Malformed PEM header");
      if(position == 0)
         label += (char)b;
      }

   Pipe base64(new Base64_Decoder);
   base64.start_msg();

   const std::string PEM_TRAILER = "-----END " + label + "-----";
   position = 0;
   while(position != PEM_TRAILER.length())
      {
      byte b;
      if(!source.read_byte(b))
         throw Decoding_Error("PEM: No PEM trailer found");
      if(b == PEM_TRAILER[position])
         position++;
      else if(position)
         throw Decoding_Error("PEM: Malformed PEM trailer");

      if(position == 0)
         base64.write(b);
      }
   base64.end_msg();
   return base64.read_all();
   }

}
