/*************************************************
* Modular Reducer Source File                    *
* (C) 1999-2003 The Botan Project                *
*************************************************/

#include <botan/reducer.h>
#include <botan/barrett.h>

namespace Botan {

/*************************************************
* Construct a ModularReducer                     *
*************************************************/
ModularReducer::ModularReducer(const BigInt& n) : modulus(n)
   {
   if(modulus < 0)
      throw Invalid_Argument("ModularReducer: modulus must be positive");
   if(modulus.size() > 8 && !power_of_2(modulus.size()))
      modulus.grow_reg((1 << high_bit(modulus.size())) - modulus.size());
   }

/*************************************************
* Modular Multiplication                         *
*************************************************/
BigInt ModularReducer::multiply(const BigInt& a, const BigInt& b) const
   {
   return reduce(a * b);
   }

/*************************************************
* Modular Squaring                               *
*************************************************/
BigInt ModularReducer::square(const BigInt& a) const
   {
   return multiply(a, a);
   }

/*************************************************
* Get a modular reducer for n                    *
*************************************************/
ModularReducer* get_reducer(const BigInt& n)
   {
   return new BarrettReducer(n);
   }

}
